@@ -271,6 +271,8 @@ pub struct GameInfo
     pub frame_interpolation_cap : Option<f64>,
     /// Whether to display the mouse cursor when it is over the window.
     pub cursor_visible : bool,
+    /// Whether unused resources should be automatically unloaded between scenes.
+    pub auto_unload : bool,
     /// Whether the computer should automatically go to sleep while the application is
     ///  running. Useful for editors and other utilities.
     pub allow_system_sleep : bool,
@@ -309,6 +311,7 @@ impl Default for GameInfo
             fullscreen : false,
             kiosk_mode : false,
             cursor_visible : true,
+            auto_unload : true,
             allow_system_sleep : false,
             low_energy_app : false,
             target_framerate : 60.0,
@@ -466,6 +469,7 @@ pub struct GameControl
     interpolation_cap : Option<f64>,
     low_energy_app : bool,
     low_energy_timer : u32,
+    auto_unload : bool,
     input : Input,
     renderer : Renderer,
     audio : Box<dyn Audio>,
@@ -566,6 +570,7 @@ impl GameControl
             interpolation_cap : gameinfo.frame_interpolation_cap,
             low_energy_app : gameinfo.low_energy_app,
             low_energy_timer : 0,
+            auto_unload : gameinfo.auto_unload,
             input,
             renderer,
             audio,
@@ -703,7 +708,7 @@ impl GameControl
     /// Returns a [DataId] for the data object with the given package and name. Avoid calling this repeatedly,
     ///  as there is a performance penalty to doing so. Please store the [DataId] where it can be used later.
     ///  Panics if the load failed.
-    pub fn lookup_res<D : DataObject + 'static>(&mut self, package_name : &str, pathname : &str) -> DataId
+    pub fn lookup_res<D : DataObject + 'static>(&mut self, package_name : &str, pathname : &str) -> DataId<D>
     {
         let mut store = self.resources.store_mut::<D>();
 
@@ -758,12 +763,22 @@ impl GameControl
     {
         let next_scene = Box::new(Scene::<T>::new());
 
+        if self.auto_unload
+        {
+            self.resources.unload_unused();
+        }
+
         self.next_scene = next_scene;
         self.next_scene_waiting = true;
     }
     
     pub fn goto_constructed_scene(&mut self, next_scene : Box<dyn BaseScene>)
     {
+        if self.auto_unload
+        {
+            self.resources.unload_unused();
+        }
+
         self.next_scene = next_scene;
         self.next_scene_waiting = true;
     }
