@@ -222,7 +222,7 @@ pub struct DrawBackgroundColorInfo
     pub g : f32,
     pub b : f32,
     pub alpha : f32,
-    pub shader_id : DataId
+    pub shader_id : DataId<Shader>
 }
 
 impl Default for DrawBackgroundColorInfo
@@ -231,11 +231,11 @@ impl Default for DrawBackgroundColorInfo
     {
         DrawBackgroundColorInfo
         {
-            r: 0.0,
-            g: 0.0,
-            b: 0.0,
-            alpha: 1.0,
-            shader_id: 0
+            r : 0.0,
+            g : 0.0,
+            b : 0.0,
+            alpha : 1.0,
+            shader_id : DataId::new(0)
         }
     }
 }
@@ -268,8 +268,8 @@ pub type SliceId = usize;
 /// Information on how to draw a sprite
 pub struct DrawSpriteInfo
 {
-    pub sheet_id : DataId,
-    pub shader_id : DataId,
+    pub sheet_id : DataId<Sheet>,
+    pub shader_id : DataId<Shader>,
     pub x : f32,
     pub y : f32,
     pub angle : f32,
@@ -286,8 +286,8 @@ pub struct DrawSpriteInfo
 /// Information on how to draw a sprite (raw form with just transform and texture coordinates)
 pub struct DrawSpriteRawInfo
 {
-    pub sheet_id : DataId,
-    pub shader_id : DataId,
+    pub sheet_id : DataId<Sheet>,
+    pub shader_id : DataId<Shader>,
     pub transform : DrawTransform,
     pub transform_offset_x : f32,
     pub transform_offset_y : f32,
