@@ -187,13 +187,13 @@ impl GpuSheetPreparer
     }
 
     #[cfg(feature = "imgui_opengl")]
-    fn update_imgui_texture(&mut self, id : DataId, handle : &mut GpuSheetHandle)
+    fn update_imgui_texture(&mut self, id : DataId<Sheet>, handle : &mut GpuSheetHandle)
     {
         let mut imgui_borrow = self.imgui_renderer.borrow_mut();
 
         if let Some(imgui_renderer) = imgui_borrow.as_mut()
         {
-            imgui_renderer.textures().replace(imgui::TextureId::from(id), imgui_glium_renderer::Texture
+            imgui_renderer.textures().replace(imgui::TextureId::from(id.index()), imgui_glium_renderer::Texture
             {
                 texture : handle.texture.clone(),
                 sampler : SamplerBehavior
@@ -238,7 +238,7 @@ impl GpuSheetPreparer
 impl DataPreparer<Sheet, GpuSheetHandle> for GpuSheetPreparer
 {
     #[allow(unused_variables)]
-    fn prepare(&mut self, data : &mut Sheet, id : DataId) -> GpuSheetHandle
+    fn prepare(&mut self, data : &mut Sheet, id : DataId<Sheet>) -> GpuSheetHandle
     {
         #[allow(unused_mut)]
         let mut handle = self.make_sheet_handle(data);
@@ -252,7 +252,7 @@ impl DataPreparer<Sheet, GpuSheetHandle> for GpuSheetPreparer
     }
 
     #[allow(unused_variables)]
-    fn unprepare(&mut self, _prepared : &mut GpuSheetHandle, id : DataId)
+    fn unprepare(&mut self, _prepared : &mut GpuSheetHandle, id : DataId<Sheet>)
     {
         #[cfg(feature = "imgui_opengl")]
         {
@@ -260,7 +260,7 @@ impl DataPreparer<Sheet, GpuSheetHandle> for GpuSheetPreparer
 
             if let Some(imgui_renderer) = imgui_borrow.as_mut()
             {
-                imgui_renderer.textures().replace(imgui::TextureId::from(id), imgui_glium_renderer::Texture
+                imgui_renderer.textures().replace(imgui::TextureId::from(id.index()), imgui_glium_renderer::Texture
                 {
                     texture : self.default_texture.clone(),
                     sampler : SamplerBehavior
@@ -276,7 +276,7 @@ impl DataPreparer<Sheet, GpuSheetHandle> for GpuSheetPreparer
     }
 
     #[allow(unused_variables)]
-    fn reprepare(&mut self, data : &mut Sheet, prepared : &mut GpuSheetHandle, id : DataId)
+    fn reprepare(&mut self, data : &mut Sheet, prepared : &mut GpuSheetHandle, id : DataId<Sheet>)
     {
         if data.texture_dirty
         {
@@ -319,7 +319,7 @@ struct GpuShaderPreparer
 
 impl DataPreparer<Shader, GpuShaderHandle> for GpuShaderPreparer
 {
-    fn prepare(&mut self, data : &mut Shader, _id : DataId) -> GpuShaderHandle
+    fn prepare(&mut self, data : &mut Shader, _id : DataId<Shader>) -> GpuShaderHandle
     {
         let program = make_program(&self.display.borrow(),
             &modify_shader_header(include_str!("shader/vert_standard_140.glsl")),
@@ -330,7 +330,7 @@ impl DataPreparer<Shader, GpuShaderHandle> for GpuShaderPreparer
             program
         }
     }
-    fn unprepare(&mut self, _prepared : &mut GpuShaderHandle, _id : DataId)
+    fn unprepare(&mut self, _prepared : &mut GpuShaderHandle, _id : DataId<Shader>)
     {
         
     }
@@ -340,8 +340,8 @@ impl DataPreparer<Shader, GpuShaderHandle> for GpuShaderPreparer
 struct InstanceBufferParam
 {
     op : RendererOp,
-    sheet : DataId,
-    shader : DataId
+    sheet : DataId<Sheet>,
+    shader : DataId<Shader>
 }
 
 fn make_program(display : &SDL2Facade, vertex_shader : &str, fragment_shader : &str) -> Result<Program, ProgramCreationError>
@@ -424,13 +424,13 @@ impl GlDrawControl
         
         let mut last_drawn : usize = 0;
         let mut current_op = RendererOp::BackgroundColor;
-        let mut current_sheet : DataId = 0;
-        let mut current_shader : DataId = 0;
+        let mut current_sheet = DataId::new(0);
+        let mut current_shader = DataId::new(0);
         
         // I can't belive Rust is making me do this
         let buffer_params = self.instance_buffer_param.clone();
         
-        let mut do_draw = |first : usize, last : usize, sheet : DataId, shader : DataId, op : RendererOp|
+        let mut do_draw = |first : usize, last : usize, sheet : DataId<Sheet>, shader : DataId<Shader>, op : RendererOp|
         {
             if first >= last
             {
@@ -732,7 +732,7 @@ impl DrawControl for GlDrawControl
         let mut texture_y = 0.0;
         let mut texture_w = 1.0;
         let mut texture_h = 1.0;
-        let mut sheet_id = 0;
+        let mut sheet_id = DataId::new(0);
 
         transform.translate(draw_info.x, draw_info.y);
         if draw_info.angle != 0.0
@@ -835,7 +835,7 @@ impl DrawControl for GlDrawControl
         self.instance_buffer_param[self.current_index] = InstanceBufferParam
         {
             op : RendererOp::BackgroundColor,
-            sheet : 0,
+            sheet : DataId::new(0),
             shader : draw_info.shader_id
         };
         
