@@ -6,6 +6,8 @@ use bit_set::BitSet;
 use bitarray_set::BitArraySet;
 use smallvec::SmallVec;
 
+use strum::EnumCount;
+use strum_macros::{EnumCount, EnumString};
 use generic_array::{GenericArray, sequence::GenericSequence};
 use generic_array::typenum::U1024;
 use typenum::Unsigned;
@@ -35,6 +37,58 @@ use std::marker::PhantomData;
 type ComponentTypeMax = U1024;
 
 
+#[macro_export] macro_rules! spawnables
+{
+    [$name:ident; $($item:ident $(: $spawnstruct:ty)?),*] =>
+    {
+        #[derive(Copy, Clone, keeshond::crate_reexport::strum_macros::EnumString, keeshond::crate_reexport::strum_macros::EnumCount)]
+        #[strum(crate = "keeshond::crate_reexport::strum")]
+        pub enum $name
+        {
+            $( $item, )*
+        }
+
+        impl Into<usize> for $name
+        {
+            fn into(self) -> usize
+            {
+                self as usize
+            }
+        }
+
+        impl keeshond::scene::SpawnableEnum for $name
+        {
+            fn spawn(&self, spawn : &mut SpawnControl, game : &mut GameControl,
+                x : f64, y : f64, args : &DynArgList)
+            {
+                match &self
+                {
+                    $( $name::$item =>
+                    {
+                        $(
+                            <$spawnstruct>::spawn(spawn, game, x, y, args);
+                        )?
+                    })*
+                }
+            }
+        }
+
+        const _ : () =
+        {
+            fn assert_spawnable_config<T : keeshond::scene::SpawnableConfig>() {}
+            fn assert_all()
+            {
+                $(
+                    //$item
+                    $(
+                        assert_spawnable_config::<$spawnstruct>();
+                    )?
+                )*
+            }
+        };
+    }
+}
+
 #[macro_export] macro_rules! darg
 {
     [$( $item:expr ),*] =>
@@ -89,7 +143,7 @@ impl Entity
     /// Returns this Entity's identifier, used for indexing operations. ID values may be reused
     /// with other Entities, though only one Entity with a given ID may be alive at a time.
     #[inline(always)]
-    pub fn get_id(&self) -> usize
+    pub fn id(&self) -> usize
     {
         self.id
     }
@@ -98,7 +152,7 @@ impl Entity
     /// is removed and put back into the pool for reuse. When querying based on Entity,
     /// both the id and generation must match. This prevents the use of stale Entity handlers.
     #[inline(always)]
-    pub fn get_generation(&self) -> u64
+    pub fn generation(&self) -> u64
     {
         self.generation
     }
@@ -172,6 +226,8 @@ pub trait ThinkerSystem<T : SceneType + 'static>
 {
     #[allow(unused_variables)]
     fn start(&mut self, args : ThinkerArgs<T>) {}
+    #[allow(unused_variables)]
+    fn start_late(&mut self, args : ThinkerArgs<T>) {}
     fn think(&mut self, args : ThinkerArgs<T>);
     #[allow(unused_variables)]
     fn end(&mut self, args : ThinkerArgs<T>) {}
@@ -468,12 +524,26 @@ impl From<Vec<DynArg>> for DynArgList
     }
 }
 
+pub trait SpawnableEnum : Copy + Clone + FromStr + EnumCount + Into<usize>
+{
+    #[allow(unused_variables)]
+    fn spawn(&self, spawn : &mut SpawnControl, game : &mut GameControl,
+             x : f64, y : f64, args : &DynArgList) {}
+}
+
+pub trait SpawnableConfig
+{
+    fn spawn(spawn : &mut SpawnControl, game : &mut GameControl,
+                 x : f64, y : f64, args : &DynArgList);
+}
+
 
 trait ComponentStoreBase : Downcast
 {
     fn add_entity_boxed(&mut self, entity : &Entity, component : Box<dyn Component>) -> bool;
     fn preview_remove_entity(&mut self, entity : &Entity) -> bool;
     fn do_remove_entity(&mut self, entity : &Entity);
+    fn purge_entities(&mut self);
 }
 impl_downcast!(ComponentStoreBase);
 
@@ -491,10 +561,8 @@ impl ComponentStoreBase for NullComponentStore
         false
     }
 
-    fn do_remove_entity(&mut self, _entity: &Entity)
-    {
-
-    }
+    fn do_remove_entity(&mut self, _entity: &Entity) {}
+    fn purge_entities(&mut self) {}
 }
 
 /// Stores all instances of a [Component] for a given type, indexed based on the [Entities](Entity)
@@ -502,10 +570,11 @@ impl ComponentStoreBase for NullComponentStore
 #[derive(Debug)]
 pub struct ComponentStore<C : Component + 'static>
 {
-    component_list : Vec<(C, u64, usize)>,
+    component_list : Vec<(C, Entity)>,
     removed_components : BitSet<u32>,
     entity_mapping : Vec<Option<usize>>,
     singleton_item : Option<C>,
+    need_purge : bool,
     ordered_removal : bool
 }
 
@@ -519,6 +588,7 @@ impl<C : Component + 'static> ComponentStore<C>
             removed_components : BitSet::new(),
             entity_mapping : Vec::with_capacity(capacity_hint),
             singleton_item : None,
+            need_purge : false,
             ordered_removal
         }
     }
@@ -534,7 +604,7 @@ impl<C : Component + 'static> ComponentStore<C>
         if self.entity_mapping[entity.id].is_none() || self.removed_components.contains(entity.id)
         {
             self.entity_mapping[entity.id] = Some(self.component_list.len());
-            self.component_list.push((component, entity.generation, entity.id));
+            self.component_list.push((component, entity.clone()));
             self.removed_components.remove(entity.id);
 
             return true;
@@ -556,9 +626,9 @@ impl<C : Component + 'static> ComponentStore<C>
         
         if let Some(index) = self.entity_mapping[entity.id]
         {
-            let (component, generation, _) = &self.component_list[index];
+            let (component, other_entity) = &self.component_list[index];
             
-            if *generation != entity.generation
+            if other_entity.generation != entity.generation
             {
                 return None;
             }
@@ -580,9 +650,9 @@ impl<C : Component + 'static> ComponentStore<C>
         
         if let Some(index) = self.entity_mapping[entity.id]
         {
-            let (component, generation, _) = &mut self.component_list[index];
+            let (component, other_entity) = &mut self.component_list[index];
             
-            if *generation != entity.generation
+            if other_entity.generation != entity.generation
             {
                 return None;
             }
@@ -592,7 +662,10 @@ impl<C : Component + 'static> ComponentStore<C>
         
         None
     }
-    
+
+    /// Returns a clone of the [Component] for the given [Entity]. This is useful
+    ///  for getting around issues with the Rust borrow checker preventing you from accessing more
+    ///  than one component in the store at a time.
     pub fn clone_from_entity(&self, entity : &Entity) -> Option<C> where C : Clone
     {
         if let Some(component) = self.get_entity(&entity)
@@ -602,7 +675,10 @@ impl<C : Component + 'static> ComponentStore<C>
         
         None
     }
-    
+
+    /// Sets the [Component] for the given [Entity] in the store. Useful to "commit" changes
+    ///  to a component that you cloned previously with [clone_from_entity()]. Returns true
+    ///  if there is a component for the given entity.
     pub fn set_entity(&mut self, entity : &Entity, component : C) -> bool
     {
         if entity.id >= self.entity_mapping.len()
@@ -612,9 +688,9 @@ impl<C : Component + 'static> ComponentStore<C>
         
         if let Some(index) = self.entity_mapping[entity.id]
         {
-            let (old_component, generation, _) = &mut self.component_list[index];
+            let (old_component, other_entity) = &mut self.component_list[index];
             
-            if *generation != entity.generation
+            if other_entity.generation != entity.generation
             {
                 return false;
             }
@@ -626,6 +702,48 @@ impl<C : Component + 'static> ComponentStore<C>
         
         false
     }
+
+    /// Returns an immutable reference to the "n-th" [Component] in the store. This is primarily
+    ///  useful for when you want to get the first, last, or random component, etc. The index
+    ///  you use here should not be cached, as the components may move around in the store.
+    ///  In most cases you should use [get_entity()] instead.
+    pub fn get_nth(&self, index : usize) -> Option<(Entity, &C)>
+    {
+        match self.component_list.get(index)
+        {
+            None => { None }
+            Some((component, entity)) =>
+            {
+                Some((entity.clone(), component))
+            }
+        }
+    }
+
+    /// Returns a mutable reference to the "n-th" [Component] in the store. This is primarily
+    ///  useful for when you want to get the first, last, or random component, etc. The index
+    ///  you use here should not be cached, as the components may move around in the store.
+    ///  In most cases you should use [get_entity_mut()] instead.
+    pub fn get_nth_mut(&mut self, index : usize) -> Option<(Entity, &mut C)>
+    {
+        match self.component_list.get_mut(index)
+        {
+            None => { None }
+            Some((component, entity)) =>
+            {
+                Some((entity.clone(), component))
+            }
+        }
+    }
+
+    pub fn first(&self) -> Option<(Entity, &C)>
+    {
+        self.get_nth(0)
+    }
+
+    pub fn first_mut(&mut self) -> Option<(Entity, &mut C)>
+    {
+        self.get_nth_mut(0)
+    }
     
     /// Returns the number of [Entities](Entity) with this [Component]
     pub fn count(&self) -> usize
@@ -643,44 +761,19 @@ impl<C : Component + 'static> ComponentStore<C>
     {
         self.component_list.sort_by(|a, b|
         {
-            let (component_a, _, _) = a;
-            let (component_b, _, _) = b;
+            let (component_a, _) = a;
+            let (component_b, _) = b;
             compare(component_a, component_b)
         });
         
         for i in 0..self.component_list.len()
         {
-            let (_, _, id) = &self.component_list[i];
+            let (_, entity) = &self.component_list[i];
             
-            self.entity_mapping[*id] = Some(i);
+            self.entity_mapping[entity.id] = Some(i);
         }
     }
     
-    fn purge_entities(&mut self)
-    {
-        if self.removed_components.is_empty()
-        {
-            return;
-        }
-
-        let mut set = BitSet::new();
-
-        std::mem::swap(&mut set, &mut self.removed_components);
-
-        self.component_list.retain(|(_, _, id)| !set.contains(*id));
-
-        std::mem::swap(&mut set, &mut self.removed_components);
-        
-        for i in 0..self.component_list.len()
-        {
-            let (_, _, id) = &self.component_list[i];
-            
-            self.entity_mapping[*id] = Some(i);
-        }
-
-        self.removed_components.clear();
-    }
-    
     /// Returns a [ComponentIter] that iterates through all [Components](Component)
     /// in the store. Typically used in a [DrawerSystem].
     pub fn iter(&self) -> ComponentIter<C>
@@ -692,8 +785,6 @@ impl<C : Component + 'static> ComponentStore<C>
     /// in the store. Typically used in a [ThinkerSystem].
     pub fn iter_mut(&mut self) -> ComponentIterMut<C>
     {
-        self.purge_entities();
-
         ComponentIterMut::<C>::new(self)
     }
     
@@ -751,9 +842,9 @@ impl<C : Component + 'static> ComponentStoreBase for ComponentStore<C>
 
         if let Some(index) = self.entity_mapping[entity.id]
         {
-            let (_, generation, _) = &mut self.component_list[index];
+            let (_, other_entity) = &mut self.component_list[index];
 
-            if *generation != entity.generation
+            if other_entity.generation != entity.generation
             {
                 return false;
             }
@@ -772,13 +863,14 @@ impl<C : Component + 'static> ComponentStoreBase for ComponentStore<C>
             {
                 self.entity_mapping[entity.id] = None;
                 self.removed_components.insert(entity.id);
+                self.need_purge = true;
             }
             else
             {
                 if !self.component_list.is_empty()
                 {
-                    let (_, _, swap_id) = self.component_list[self.component_list.len() - 1];
-                    self.entity_mapping[swap_id] = self.entity_mapping[entity.id];
+                    let (_, other_entity) = &self.component_list[self.component_list.len() - 1];
+                    self.entity_mapping[other_entity.id] = self.entity_mapping[entity.id];
                 }
 
                 self.component_list.swap_remove(index);
@@ -790,6 +882,32 @@ impl<C : Component + 'static> ComponentStoreBase for ComponentStore<C>
             panic!("Tried to remove unmapped entity {}", entity);
         }
     }
+
+    fn purge_entities(&mut self)
+    {
+        if !self.need_purge
+        {
+            return;
+        }
+
+        let mut set = BitSet::new();
+
+        std::mem::swap(&mut set, &mut self.removed_components);
+
+        self.component_list.retain(|(_, entity)| !set.contains(entity.id));
+
+        std::mem::swap(&mut set, &mut self.removed_components);
+
+        for i in 0..self.component_list.len()
+        {
+            let (_, entity) = &self.component_list[i];
+
+            self.entity_mapping[entity.id] = Some(i);
+        }
+
+        self.removed_components.clear();
+        self.need_purge = false;
+    }
 }
 
 
@@ -798,7 +916,7 @@ impl<C : Component + 'static> ComponentStoreBase for ComponentStore<C>
 
 pub struct ComponentIter<'a, C : Component + 'static>
 {
-    data_iter : std::slice::Iter<'a, (C, u64, usize)>,
+    data_iter : std::slice::Iter<'a, (C, Entity)>,
 }
 
 impl<'a, C : Component + 'static> ComponentIter<'a, C>
@@ -815,11 +933,9 @@ impl<'a, C : Component + 'static> Iterator for ComponentIter<'a, C>
     
     fn next(&mut self) -> Option<(Entity, &'a C)>
     {
-        if let Some((component, generation, id)) = self.data_iter.next()
+        if let Some((component, entity)) = self.data_iter.next()
         {
-            let entity = Entity::new(*id, *generation);
-                
-            return Some((entity, component));
+            return Some((entity.clone(), component));
         }
         
         None
@@ -829,7 +945,7 @@ impl<'a, C : Component + 'static> Iterator for ComponentIter<'a, C>
 
 pub struct ComponentIterMut<'a, C : Component + 'static>
 {
-    data_iter : std::slice::IterMut<'a, (C, u64, usize)>,
+    data_iter : std::slice::IterMut<'a, (C, Entity)>,
 }
 
 impl<'a, C : Component + 'static> ComponentIterMut<'a, C>
@@ -846,11 +962,9 @@ impl<'a, C : Component + 'static> Iterator for ComponentIterMut<'a, C>
     
     fn next(&mut self) -> Option<(Entity, &'a mut C)>
     {
-        if let Some((component, generation, id)) = self.data_iter.next()
+        if let Some((component, entity)) = self.data_iter.next()
         {
-            let entity = Entity::new(*id, *generation);
-                
-            return Some((entity, component));
+            return Some((entity.clone(), component));
         }
         
         None
@@ -1028,8 +1142,8 @@ impl ComponentControl
         let data = self.data.try_borrow().expect("ComponentStore data already in use!");
         let index = data.component_id_to_index[&wanted_type];
 
-        return Some(Ref::map(self.component_stores[index].borrow(),
-                             |b| b.downcast_ref::<ComponentStore<C>>().expect("ComponentStore borrow failed!")));
+        Some(Ref::map(self.component_stores[index].borrow(),
+                             |b| b.downcast_ref::<ComponentStore<C>>().expect("ComponentStore borrow failed!")))
     }
     
     pub fn try_store_mut<C : Component + 'static>(&self) -> Option<RefMut<ComponentStore<C>>>
@@ -1049,15 +1163,8 @@ impl ComponentControl
         let data = self.data.try_borrow().expect("ComponentStore data already in use!");
         let index = data.component_id_to_index[&wanted_type];
 
-        let mut borrow = Some(RefMut::map(self.component_stores[index].borrow_mut(),
-                           |b| b.downcast_mut::<ComponentStore<C>>().expect("ComponentStore borrow failed!")));
-
-        if let Some(store) = &mut borrow
-        {
-            store.purge_entities();
-        }
-
-        borrow
+        Some(RefMut::map(self.component_stores[index].borrow_mut(),
+                           |b| b.downcast_mut::<ComponentStore<C>>().expect("ComponentStore borrow failed!")))
     }
 }
 
@@ -1302,7 +1409,7 @@ impl<T : SceneType + 'static> SceneConfig<T>
 /// and which systems this scene should include.
 pub trait SceneType
 {
-    type SpawnableIdType : Copy + FromStr + 'static;
+    type SpawnableIdType : SpawnableEnum + 'static;
     
     fn new() -> Self where Self : Sized;
     #[allow(unused_variables)]
@@ -1317,9 +1424,25 @@ pub struct NullSceneType
 
 }
 
+#[derive(Copy, Clone, EnumString, EnumCount)]
+pub enum NullSpawnable {}
+
+impl Into<usize> for NullSpawnable
+{
+    fn into(self) -> usize
+    {
+        self as usize
+    }
+}
+
+impl SpawnableEnum for NullSpawnable
+{
+
+}
+
 impl SceneType for NullSceneType
 {
-    type SpawnableIdType = u32;
+    type SpawnableIdType = NullSpawnable;
     
     fn new() -> Self where Self : Sized { NullSceneType {} }
     fn config(&mut self, _game: &mut GameControl) -> SceneConfig<Self> { SceneConfig::new() }
@@ -1351,12 +1474,18 @@ pub trait BaseScene
 }
 
 
+struct EntityRecord
+{
+    component_bits : BitArraySet<u32, ComponentTypeMax>,
+    generation : u64
+}
+
 /// A world in which [Entities](Entity), [Components](Component), and [ThinkerSystems](ThinkerSystem)/[DrawerSystems](DrawerSystem) reside.
 pub struct Scene<T : SceneType + 'static>
 {
     components : ComponentControl,
     control : SceneControl<T>,
-    entity_components : Vec<Option<(BitArraySet<u32, ComponentTypeMax>, u64)>>,
+    entity_records : Vec<EntityRecord>,
     thinker_systems : Vec<Box<dyn ThinkerSystem<T>>>,
     drawer_systems : Vec<Box<dyn DrawerSystem>>,
     #[cfg(feature = "imgui_base")]
@@ -1364,6 +1493,8 @@ pub struct Scene<T : SceneType + 'static>
     num_entities : usize,
     num_components : usize,
     component_callbacks : GenericArray<ComponentCallbacks, ComponentTypeMax>,
+    need_purge : bool,
+    stores_to_purge : BitArraySet<u32, ComponentTypeMax>,
     init_done : bool,
     deinit_done : bool
 }
@@ -1437,10 +1568,14 @@ impl<T : SceneType + 'static> BaseScene for Scene<T>
 
         for system in list.iter_mut()
         {
-            system.start(ThinkerArgs
-            {
-                components : &mut self.components, scene : &mut self.control, game
-            });
+            system.start(ThinkerArgs { components : &mut self.components, scene : &mut self.control, game });
+
+            self.do_deferred_actions();
+        }
+
+        for system in list.iter_mut()
+        {
+            system.start_late(ThinkerArgs { components : &mut self.components, scene : &mut self.control, game });
 
             self.do_deferred_actions();
         }
@@ -1567,18 +1702,20 @@ impl<T : SceneType + 'static> Scene<T>
 
         Scene
         {
-            components: ComponentControl::new(),
-            control: SceneControl::<T>::new(),
-            entity_components: Vec::new(),
-            thinker_systems: Vec::new(),
-            drawer_systems: Vec::new(),
+            components : ComponentControl::new(),
+            control : SceneControl::<T>::new(),
+            entity_records : Vec::new(),
+            thinker_systems : Vec::new(),
+            drawer_systems : Vec::new(),
             #[cfg(feature = "imgui_base")]
-            imgui_system: Box::new(NullImGuiSystem { _phantom : PhantomData }),
-            num_entities: 0,
-            num_components: 0,
+            imgui_system : Box::new(NullImGuiSystem { _phantom : PhantomData }),
+            num_entities : 0,
+            num_components : 0,
             component_callbacks,
-            init_done: false,
-            deinit_done: false,
+            need_purge : false,
+            stores_to_purge : BitArraySet::new(),
+            init_done : false,
+            deinit_done : false,
         }
     }
     
@@ -1606,7 +1743,7 @@ impl<T : SceneType + 'static> Scene<T>
     {
         let wanted_type = TypeId::of::<C>();
         
-        if entity.id >= self.entity_components.len()
+        if entity.id >= self.entity_records.len()
         {
             return false;
         }
@@ -1615,76 +1752,70 @@ impl<T : SceneType + 'static> Scene<T>
         {
             self.components.register_component_type::<C>();
         }
-        
-        if let Some((component_bits, generation)) = &mut self.entity_components[entity.id]
+
+        let record = &mut self.entity_records[entity.id];
+
+        if entity.generation != record.generation
         {
-            if entity.generation != *generation
-            {
-                return false;
-            }
-
-            let data = self.components.data.try_borrow_mut().expect("ComponentStore data already in use!");
-            let index = data.component_id_to_index[&wanted_type];
-
-            let mut component_store = RefMut::map(self.components.component_stores[index].borrow_mut(),
-                        |b| b.downcast_mut::<ComponentStore<C>>().expect("ComponentStore borrow failed!"));
-
-            let result = component_store.add_entity(entity, component);
-
-            std::mem::drop(data);
-            std::mem::drop(component_store);
-
-            if result
-            {
-                self.num_components += 1;
-                component_bits.insert(index);
-
-                self.component_callbacks[index].call_add_component_callback(entity, &mut self.components);
-            }
-
-            return result;
+            return false;
         }
 
-        false
+        let data = self.components.data.try_borrow_mut().expect("ComponentStore data already in use!");
+        let index = data.component_id_to_index[&wanted_type];
+
+        let mut component_store = RefMut::map(self.components.component_stores[index].borrow_mut(),
+                    |b| b.downcast_mut::<ComponentStore<C>>().expect("ComponentStore borrow failed!"));
+
+        let result = component_store.add_entity(entity, component);
+
+        std::mem::drop(data);
+        std::mem::drop(component_store);
+
+        if result
+        {
+            self.num_components += 1;
+            record.component_bits.insert(index);
+
+            self.component_callbacks[index].call_add_component_callback(entity, &mut self.components);
+        }
+
+        return result;
     }
     
     fn add_component_boxed(&mut self, entity : &Entity, component : Box<dyn Component>, component_type : TypeId) -> bool
     {
-        if entity.id >= self.entity_components.len()
+        if entity.id >= self.entity_records.len()
         {
             return false;
         }
         
         self.ensure_boxed_component_registered(component_type);
         
-        if let Some((component_bits, generation)) = &mut self.entity_components[entity.id]
+        let record = &mut self.entity_records[entity.id];
+
+        if entity.generation != record.generation
         {
-            if entity.generation != *generation
-            {
-                return false;
-            }
-
-            let data = self.components.data.try_borrow_mut().expect("ComponentStore data already in use!");
-            let index = data.component_id_to_index[&component_type];
-
-            let mut component_store = self.components.component_stores[index].borrow_mut();
-            let result = component_store.add_entity_boxed(entity, component);
-
-            std::mem::drop(data);
-            std::mem::drop(component_store);
-            
-            if result
-            {
-                self.num_components += 1;
-                component_bits.insert(index);
-
-                self.component_callbacks[index].call_add_component_callback(entity, &mut self.components);
-            }
-
-            return result;
+            return false;
         }
-        
-        false
+
+        let data = self.components.data.try_borrow_mut().expect("ComponentStore data already in use!");
+        let index = data.component_id_to_index[&component_type];
+
+        let mut component_store = self.components.component_stores[index].borrow_mut();
+        let result = component_store.add_entity_boxed(entity, component);
+
+        std::mem::drop(data);
+        std::mem::drop(component_store);
+
+        if result
+        {
+            self.num_components += 1;
+            record.component_bits.insert(index);
+
+            self.component_callbacks[index].call_add_component_callback(entity, &mut self.components);
+        }
+
+        return result;
     }
     
     /// Detaches a [Component] from the given [Entity]. Returns true if successful.
@@ -1697,41 +1828,41 @@ impl<T : SceneType + 'static> Scene<T>
     {
         let data = self.components.data.try_borrow_mut().expect("ComponentStore data already in use!");
 
-        if entity.id >= self.entity_components.len() || !data.component_id_to_index.contains_key(&component_type)
+        if entity.id >= self.entity_records.len() || !data.component_id_to_index.contains_key(&component_type)
         {
             return false;
         }
         
-        if let Some((component_bits, generation)) = &mut self.entity_components[entity.id]
+        let record = &mut self.entity_records[entity.id];
+
+        if entity.generation != record.generation
         {
-            if entity.generation != *generation
-            {
-                return false;
-            }
-
-            let index = data.component_id_to_index[&component_type];
-
-            let mut component_store = self.components.component_stores[index].borrow_mut();
-            let result = component_store.preview_remove_entity(entity);
-
-            std::mem::drop(data);
-            
-            if result
-            {
-                component_store.do_remove_entity(entity);
-
-                std::mem::drop(component_store);
-
-                self.num_components -= 1;
-                component_bits.remove(index);
-
-                self.component_callbacks[index].call_remove_component_callback(entity, &mut self.components);
-            }
-            
-            return result;
+            return false;
         }
-        
-        false
+
+        let index = data.component_id_to_index[&component_type];
+
+        let mut component_store = self.components.component_stores[index].borrow_mut();
+        let result = component_store.preview_remove_entity(entity);
+
+        std::mem::drop(data);
+
+        if result
+        {
+            component_store.do_remove_entity(entity);
+
+            std::mem::drop(component_store);
+
+            self.num_components -= 1;
+            record.component_bits.remove(index);
+            self.stores_to_purge.insert(index);
+            self.need_purge = true;
+            self.control.deferred_flag = true;
+
+            self.component_callbacks[index].call_remove_component_callback(entity, &mut self.components);
+        }
+
+        return result;
     }
     
     /// Creates a new [Entity] within the scene, returning its handler.
@@ -1746,17 +1877,26 @@ impl<T : SceneType + 'static> Scene<T>
     
     fn add_entity_internal(&mut self, entity : &Entity)
     {
-        if entity.id < self.entity_components.len()
+        debug_assert_ne!(entity.generation, 0);
+
+        if entity.id < self.entity_records.len()
         {
-            self.entity_components[entity.id] = Some((BitArraySet::new(), entity.generation));
+            let record = &mut self.entity_records[entity.id];
+
+            record.component_bits = BitArraySet::new();
+            record.generation = entity.generation;
         }
-        else if entity.id == self.entity_components.len()
+        else if entity.id == self.entity_records.len()
         {
-            self.entity_components.insert(entity.id, Some((BitArraySet::new(), entity.generation)));
+            self.entity_records.insert(entity.id, EntityRecord
+            {
+                component_bits : BitArraySet::new(),
+                generation : entity.generation
+            });
         }
         else
         {
-            panic!("Unexpected entity ID number. Got {}, expected {} or lower", entity.id, self.entity_components.len());
+            panic!("Unexpected entity ID number. Got {}, expected {} or lower", entity.id, self.entity_records.len());
         }
         
         self.num_entities += 1;
@@ -1765,65 +1905,62 @@ impl<T : SceneType + 'static> Scene<T>
     /// Removes the [Entity] with the given handle from the scene. Returns true if successful.
     pub fn remove_entity(&mut self, entity : &Entity) -> bool
     {
-        if entity.id >= self.entity_components.len()
+        if entity.id >= self.entity_records.len()
         {
             return false;
         }
         
-        if let Some((component_bits, generation)) = &self.entity_components[entity.id]
+        let record = &mut self.entity_records[entity.id];
+
+        if entity.generation != record.generation
         {
-            if entity.generation != *generation
-            {
-                return false;
-            }
-            
-            for component_index in component_bits
-            {
-                if self.components.component_stores[component_index].borrow_mut().preview_remove_entity(&entity)
-                {
-                    self.component_callbacks[component_index].call_remove_component_callback(entity, &mut self.components);
-
-                    let mut component_store = self.components.component_stores[component_index].borrow_mut();
-
-                    component_store.do_remove_entity(&entity);
-                    self.num_components -= 1;
-                }
-            }
-            
-            self.entity_components[entity.id] = None;
-
-            let mut next_generation = entity.generation + 1;
-
-            if next_generation == 0
-            {
-                warn!("Generation overflow for entity ID {}!", entity.id);
-                next_generation = 1;
-            }
-            
-            self.control.unused_pool.push_back(Entity::new(entity.id, next_generation));
-            
-            self.num_entities -= 1;
-            
-            return true;
+            return false;
         }
-        
-        false
+
+        for component_index in &record.component_bits
+        {
+            if self.components.component_stores[component_index].borrow_mut().preview_remove_entity(&entity)
+            {
+                self.component_callbacks[component_index].call_remove_component_callback(entity, &mut self.components);
+
+                let mut component_store = self.components.component_stores[component_index].borrow_mut();
+
+                component_store.do_remove_entity(&entity);
+                self.num_components -= 1;
+                self.stores_to_purge.insert(component_index);
+                self.need_purge = true;
+                self.control.deferred_flag = true;
+            }
+        }
+
+        record.generation = 0;
+
+        let mut next_generation = entity.generation + 1;
+
+        if next_generation == 0
+        {
+            warn!("Generation overflow for entity ID {}!", entity.id);
+            next_generation = 1;
+        }
+
+        self.control.unused_pool.push_back(Entity::new(entity.id, next_generation));
+
+        self.num_entities -= 1;
+
+        return true;
     }
     
     /// Returns true if the scene has the given [Entity].
     pub fn has_entity(&self, entity : &Entity) -> bool
     {
-        if entity.id >= self.entity_components.len()
+        if entity.id >= self.entity_records.len()
         {
             return false;
         }
         
-        if let Some((_, generation)) = self.entity_components[entity.id]
-        {
-            return entity.generation == generation;
-        }
-        
-        false
+        let record = &self.entity_records[entity.id];
+
+        record.generation > 0 && entity.generation == record.generation
     }
 
     pub fn spawn(&mut self, spawnable_id : T::SpawnableIdType, game : &mut GameControl,
@@ -1838,10 +1975,12 @@ impl<T : SceneType + 'static> Scene<T>
         self.control.scene_type.spawn(&mut self.control.spawn_control, spawnable_id, game, x, y, args);
         
         let entity = self.add_entity();
-        
-        while !self.control.spawn_control.components.is_empty()
+
+        let mut components = Vec::new();
+        std::mem::swap(&mut components, &mut self.control.spawn_control.components);
+
+        for component in components.drain(..)
         {
-            let component = self.control.spawn_control.components.pop().unwrap();
             let component_type = (*component).type_id();
             self.add_component_boxed(&entity, component, component_type);
         }
@@ -1956,6 +2095,16 @@ impl<T : SceneType + 'static> Scene<T>
                 self.remove_entity(entity);
             }
         }
+
+        if self.need_purge
+        {
+            for index in &self.stores_to_purge
+            {
+                self.components.component_stores[index].borrow_mut().purge_entities();
+            }
+            self.stores_to_purge.clear();
+            self.need_purge = false;
+        }
         
         self.control.deferred_flag = false;
     }
