@@ -290,6 +290,17 @@ impl<'a, T> IntoIterator for &'a NamedItemStore<T>
     }
 }
 
+impl<'a, T> IntoIterator for &'a mut NamedItemStore<T>
+{
+    type Item = &'a mut T;
+    type IntoIter = std::slice::IterMut<'a, T>;
+
+    fn into_iter(self) -> Self::IntoIter
+    {
+        self.items.iter_mut()
+    }
+}
+
 #[derive(Deserialize, Debug)]
 struct NamedItemStoreRaw<T>
 {
